==> 4f-0.0.3.tar.gz: 4f-0.0.3/lib/FFFF.pm <==
package FFFF;
use strict;

use FFFF::Session;
use FFFF::Config;
use LocalCGI;
use InputCheck;

# -------------------------------------------------------------------------
# new
#
# -------------------------------------------------------------------------
sub new {
	my $class = shift;
	my %option = @_;

	my %form = QUERY_decode();

	my $action = ($form{_action} =~ /^([\-\w]+)$/)[0];
	my $back   = ($form{_back} =~ /^([\-\w]+)$/)[0];

	return bless {
		ACTION => uc $action,
		BACK   => uc $back,
		FORM   => \%form
	}, $class;
}

# -------------------------------------------------------------------------
# եɤ߹
#
# -------------------------------------------------------------------------
sub readconfig {
	my $self = shift;

	my $config = FFFF::Config->read();
	$self->{CONFIG} = $config->{DATA};

	my $action = $self->{BACK} || $self->{ACTION};
	$self->{CONFIG_NOW} = $self->{CONFIG}->{ACTION}->{$action};
}

# -------------------------------------------------------------------------
# åǡΥå
#
# -------------------------------------------------------------------------
sub session_check {
	my $self = shift;

	return if (ref $self->{CONFIG_NOW}->{SESSION_CHECK} ne 'ARRAY');

	my %session = $self->session_data();
	for my $action  (map {@$_} $self->{CONFIG_NOW}->{SESSION_CHECK}) {
		my $config = $self->{CONFIG}->{ACTION}->{$action};

		my $check = InputCheck->new(
			%{$config->{INPUT_CHECK}->{PLAN}}
		);

		$check->execute(%session);

		if ($check->is_error() == 1) {
			$self->{ERROR} = {
				$check->get_error(
					%{$config->{INPUT_CHECK}->{ALIAS}}
				)
			};
			$self->{CONFIG_NOW}->{TEMPLATE}->{ERROR} = $config->{TEMPLATE}->{ERROR};
			last;
		}
	}
}


# -------------------------------------------------------------------------
# ϥå
#
# -------------------------------------------------------------------------
sub input_check {
	my $self = shift;

	my $check = InputCheck->new(
		%{$self->{CONFIG_NOW}->{INPUT_CHECK}->{PLAN}}
	);

	$check->execute(
		%{$self->{FORM}}
	);

	if ($check->is_error() == 1) {
		$self->{ERROR} = {
			$check->get_error(
				%{$self->{CONFIG_NOW}->{INPUT_CHECK}->{ALIAS}}
			)
		};
	}
}

sub is_error {
	my $self = shift;

	return (ref $self->{ERROR} eq "HASH")? 1 : 0;
}


# -------------------------------------------------------------------------
# å¸
#
# TODO:
# - åطδؿƬˤ줾 SessionType None ν
#   񤤤ƤΤȤʤС
#
# -------------------------------------------------------------------------
sub session_set {
	my $self = shift;

	return if ($self->{CONFIG}->{SESSION_TYPE} eq 'NONE');

	my $session = FFFF::Session->new (
		SESSION_DIR  => $self->{CONFIG}->{SESSION_DIR},
		COOKIE_KEY   => $self->{CONFIG}->{COOKIE_NAME}
	);

	
	#
	# å򳫤Ƥ鼺Ԥ
	#
	if ($self->{ACTION} eq "" or $self->{SESSION_CREATE} == 1) {
		eval {
			$session->open();
		};
		if($@){
			$session->create();
		}
	}

	#
	# å򳫤Ԥ饨顼
	#
	else {
		$session->open() or die "Can't Open Session";
	}

	$session->cleanup(%{$self->{FORM}});
	$session->set(%{$self->{FORM}});

	$self->{SESSION} = $session;
}

sub session_data {
	my $self = shift;

	if ($self->{CONFIG}->{SESSION_TYPE} eq 'NONE') {
		return %{$self->{FORM}};
	}
	else {
		return $self->{SESSION}->get_hash();
	}
}

sub session_expire {
	my $self = shift;

	return if ($self->{CONFIG}->{SESSION_TYPE} eq 'NONE');
	return $self->{SESSION}->expire();
}

# -------------------------------------------------------------------------
# ⥸塼μ¹
#
# -------------------------------------------------------------------------
sub exec_module {
	my $self = shift;

	for my $module_obj (@{$self->{CONFIG_NOW}->{MODULE}}) {
		$module_obj->execute(
			$self->session_data()
		);
	}
}


# -------------------------------------------------------------------------
# ̤ν
#
# -------------------------------------------------------------------------
sub outputhtml {
	my $self = shift;

	#
	# Ϥ HTMLƥץ졼Ȥμ
	#
	my $template;
	if ($self->is_error() == 0) {
		$template = $self->{CONFIG_NOW}->{TEMPLATE}->{NEXT};
	}
	else {
		$template = $self->{CONFIG_NOW}->{TEMPLATE}->{ERROR};
	}
	$template ||= $self->{CONFIG}->{DEFAULT_TEMPLATE};
	$template = $self->search_template($template);

	#
	# HIDDEN_LOOP κ
	#
	my @hidden_loop;
	while (my ($name, $value) = each %{$self->{FORM}}) {
		
		#
		# name  Ƭ _ ϤޤΤϥƥѿ
		#
		next if ($name =~ /^_/);
		push(@hidden_loop, {name => $name, value => $value});
	}

	OutputHtml($template,
		$self->session_data(),
		%{$self->{CONFIG}->{DEFAULT_VALUE}},
		%{$self->{FORM}},
		%{$self->{ERROR}},
		_hidden_loop => \@hidden_loop,
	);
}

# -------------------------------------------------------------------------
# Ϥƥץ졼Ȥθ
#
# -------------------------------------------------------------------------
sub search_template {
	my $self = shift;
	my ($template) = @_;

	#
	# ե뤬̵ä search_path 򸡺
	#
	my @search = @{$self->{CONFIG}->{SEARCH_PATH}};
	if (!(-e $template) and $template =~ /^\w/) {
		for my $path (@search) {
			if (-e "$path/$template") {
				$template = "$path/$template";
				last;
			}
		}
	}
	unless (-e $template) {
		my ($package, $filename) = caller();
		die "Can't locate $template in search_path (search_path: @search, package: $package)";
	}

	return $template;
}

1;

__END__
=head CHANGES
0.0.3   2002/05/21
	- SessionType  None ΤȤνɲá
	- Ϥ줿եह٤ƤäƤ hidden_loop Ϥ褦ˤ

0.0.2   2002/05/15
	- ꤵƤʤ Action äƤ硢åå
	  顼ˤʤХ

0.0.1   2002/05/09
	- original version
=cut

==> 4f-0.0.4.tar.gz: 4f-0.0.4/lib/FFFF.pm <==
package FFFF;
use strict;

use FFFF::Session;
use FFFF::Config;
use LocalCGI;
use InputCheck;

# -------------------------------------------------------------------------
# new
#
# -------------------------------------------------------------------------
sub new {
	my $class = shift;
	my %option = @_;

	my %form = QUERY_decode();

	my $action = ($form{_action} =~ /^([\-\w]+)$/)[0];
	my $back   = ($form{_back} =~ /^([\-\w]+)$/)[0];

	return bless {
		ACTION => uc $action,
		BACK   => uc $back,
		FORM   => \%form
	}, $class;
}

# -------------------------------------------------------------------------
# եɤ߹
#
# -------------------------------------------------------------------------
sub readconfig {
	my $self = shift;

	my $config = FFFF::Config->read();
	$self->{CONFIG} = $config->{DATA};

	#
	# CONFIG_NOW 
	#
	my $action = $self->{BACK} || $self->{ACTION};
	$self->{CONFIG_NOW} = $self->{CONFIG}->{ACTION}->{$action};

	#
	# IfConfig Ÿ
	#
	$self->if_config();

}

sub if_config {
	my $self = shift;

	for my $ifconfig (@{$self->{CONFIG_NOW}->{IF_CONFIG}}) {
		my $value  = ($ifconfig->{VALUE} eq 'NULL')? "" : $ifconfig->{VALUE};
		my $key    = $ifconfig->{KEY};
		my $rule   = $ifconfig->{RULE};
		my $config = $ifconfig->{CONFIG};

		if (
			($rule eq '='  and $self->{FORM}->{$key} eq $value) or
			($rule eq '!=' and $self->{FORM}->{$key} ne $value) or
			($rule eq '>'  and $self->{FORM}->{$key} >  $value) or
			($rule eq '<'  and $self->{FORM}->{$key} <  $value) or
			($rule eq '<=' and $self->{FORM}->{$key} <= $value) or
			($rule eq '>=' and $self->{FORM}->{$key} >= $value)
		) {

			#
			# Next
			#
			if ($config->{NEXT} ne "") {
				$self->{CONFIG_NOW}->{TEMPLATE}->{NEXT} = $config->{NEXT};
			}

			#
			# InputCheck
			#
			while (my ($key, $value) = each %{$config->{INPUT_CHECK_ADD}->{PLAN}} ) {
				$self->{CONFIG_NOW}->{INPUT_CHECK}->{PLAN}->{$key} = [
					@{$self->{CONFIG_NOW}->{INPUT_CHECK}->{PLAN}->{$key}},
					@{$value}
				];
			}
			while (my ($key, $value) = each %{$config->{INPUT_CHECK_ADD}->{ALIAS}} ) {
				$self->{CONFIG_NOW}->{INPUT_CHECK}->{ALIAS}->{$key} = $value;
			}	


			#
			# Module
			#
			$self->{CONFIG_NOW}->{MODULE} = [
				@{$self->{CONFIG_NOW}->{MODULE}},
				@{$config->{MODULE}}
			];
		}
	}

}

# -------------------------------------------------------------------------
# åǡΥå
#
# -------------------------------------------------------------------------
sub session_check {
	my $self = shift;

	return if (ref $self->{CONFIG_NOW}->{SESSION_CHECK} ne 'ARRAY');

	my %session = $self->session_data();
	for my $action  (map {@$_} $self->{CONFIG_NOW}->{SESSION_CHECK}) {
		my $config = $self->{CONFIG}->{ACTION}->{$action};

		my $check = InputCheck->new(
			%{$config->{INPUT_CHECK}->{PLAN}}
		);

		$check->execute(%session);

		if ($check->is_error() == 1) {
			$self->{ERROR} = {
				$check->get_error(
					%{$config->{INPUT_CHECK}->{ALIAS}}
				)
			};
			$self->{CONFIG_NOW}->{TEMPLATE}->{ERROR} = $config->{TEMPLATE}->{ERROR};
			last;
		}
	}
}


# -------------------------------------------------------------------------
# ϥå
#
# -------------------------------------------------------------------------
sub input_check {
	my $self = shift;

	my $check = InputCheck->new(
		%{$self->{CONFIG_NOW}->{INPUT_CHECK}->{PLAN}}
	);

	$check->execute(
		%{$self->{FORM}}
	);

	if ($check->is_error() == 1) {
		$self->{ERROR} = {
			$check->get_error(
				%{$self->{CONFIG_NOW}->{INPUT_CHECK}->{ALIAS}}
			)
		};
	}
}

sub is_error {
	my $self = shift;

	return (ref $self->{ERROR} eq "HASH")? 1 : 0;
}


# -------------------------------------------------------------------------
# å¸
#
# TODO:
# - åطδؿƬˤ줾 SessionType None ν
#   񤤤ƤΤȤʤС
#
# -------------------------------------------------------------------------
sub session_set {
	my $self = shift;

	return if ($self->{CONFIG}->{SESSION_TYPE} eq 'NONE');

	my $session = FFFF::Session->new (
		SESSION_DIR  => $self->{CONFIG}->{SESSION_DIR},
		COOKIE_KEY   => $self->{CONFIG}->{COOKIE_NAME}
	);

	
	#
	# å򳫤Ƥ鼺Ԥ
	#
	if ($self->{ACTION} eq "" or $self->{SESSION_CREATE} == 1) {
		eval {
			$session->open();
		};
		if($@){
			$session->create();
		}
	}

	#
	# å򳫤Ԥ饨顼
	#
	else {
		$session->open() or die "Can't Open Session";
	}

	$session->cleanup(%{$self->{FORM}});
	$session->set(%{$self->{FORM}});

	$self->{SESSION} = $session;
}

sub session_data {
	my $self = shift;

	if ($self->{CONFIG}->{SESSION_TYPE} eq 'NONE') {
		return %{$self->{FORM}};
	}
	else {
		return $self->{SESSION}->get_hash();
	}
}

sub session_expire {
	my $self = shift;

	return if ($self->{CONFIG}->{SESSION_TYPE} eq 'NONE');
	return $self->{SESSION}->expire();
}

# -------------------------------------------------------------------------
# ⥸塼μ¹
#
# -------------------------------------------------------------------------
sub exec_module {
	my $self = shift;

	for my $module_obj (@{$self->{CONFIG_NOW}->{MODULE}}) {
		$module_obj->execute(
			$self->session_data()
		);
	}
}


# -------------------------------------------------------------------------
# ̤ν
#
# -------------------------------------------------------------------------
sub outputhtml {
	my $self = shift;

	#
	# Ϥ HTMLƥץ졼Ȥμ
	#
	my $template;
	if ($self->is_error() == 0) {
		$template = $self->{CONFIG_NOW}->{TEMPLATE}->{NEXT};
	}
	else {
		$template = $self->{CONFIG_NOW}->{TEMPLATE}->{ERROR};
	}
	$template ||= $self->{CONFIG}->{DEFAULT_TEMPLATE};
	$template = $self->search_template($template);

	#
	# HIDDEN_LOOP κ
	#
	my @hidden_loop;
	while (my ($name, $value) = each %{$self->{FORM}}) {
		
		#
		# name  Ƭ _ ϤޤΤϥƥѿ
		#
		next if ($name =~ /^_/);
		push(@hidden_loop, {name => $name, value => $value});
	}

	OutputHtml($template,
		$self->session_data(),
		%{$self->{CONFIG}->{DEFAULT_VALUE}},
		%{$self->{FORM}},
		%{$self->{ERROR}},
		_hidden_loop => \@hidden_loop,
	);
}

# -------------------------------------------------------------------------
# Ϥƥץ졼Ȥθ
#
# -------------------------------------------------------------------------
sub search_template {
	my $self = shift;
	my ($template) = @_;

	#
	# ե뤬̵ä search_path 򸡺
	#
	my @search = @{$self->{CONFIG}->{SEARCH_PATH}};
	if (!(-e $template) and $template =~ /^\w/) {
		for my $path (@search) {
			if (-e "$path/$template") {
				$template = "$path/$template";
				last;
			}
		}
	}
	unless (-e $template) {
		my ($package, $filename) = caller();
		die "Can't locate $template in search_path (search_path: @search, package: $package)";
	}

	return $template;
}

1;

__END__
=head CHANGES
0.0.4   2002/07/15
	- ͤˤäưŪѹ IfConfig 򥵥ݡ

0.0.3   2002/05/21
	- SessionType  None ΤȤνɲá
	- Ϥ줿եह٤ƤäƤ hidden_loop Ϥ褦ˤ

0.0.2   2002/05/15
	- ꤵƤʤ Action äƤ硢åå
	  顼ˤʤХ

0.0.1   2002/05/09
	- original version
=cut
